/**@name LIBRARY: unConfigCMW.ctl

@author: Herve Milcent (AB/CO)

Creation Date: 19/04/2005

Modification History:
  30/03/2014: Herve
    - IS-1656 get CMW address: polling group, remove system name
    - IS-1657 getCMW address: get cycle name and onChange flag
    - IS-1658 setCMW address: set cycle name and onChange flag
    
  25/03/2015: Herve, IS-1628, support subindex in the importation line
  
  14/10/2011: Herve
  - IS-593  unCore - import/export  stop/restart SIMULATOR/driver from import DB panel 
 
  30/05/2011: Herve
    - IS-519: Remove all dependencies on graphical object from the Import device panel in the core import 
    functions, to be able to use the unicos front-end/device check/import functions from a WCCOActrl 
    
  02/05/2011: Herve
  - IS-510: CPC 5 and CPC 6 compatibility of the export/import of the CPC device configuration with all the front-end types
  
  25/08/2010: Herve
    IS-392: do not set the config file if the section already exists in the config file
    
  19/02/2009: Herve
    - include unCMW.ctl libs
    
	13/03/2008 Quique
		- Modify the Analog Input functions check,set and export
		
	30/04/2007: Herve
		- error in export if driver not started
		
	20/03/2007: Herve
		- add extra param for CMW front-end: GQ allowed
		- export
		
	06/11/2006: Herve
		- add set address config
		- export function
		- AnalogStatus, WordStatus
		- add counter on CMW front-end
		
	12/05/2004: Herve
		- DigitalInput: delete archive on evStsReg01 if no archive on PosSt
		
	22/04/2005:
		- implement the event archive: check the event archive name
		- remove UN_CONFIG_DEFAULT_ARCHIVE_SMOOTHING and replace in libs by DPATTR_ARCH_PROC_SIMPLESM
		- allow archive in import file per device
		- add new function CMW_getFrontEndArchiveDp: return the list of Dp that will be archive for the front-end

version 1#

External Function :
  . unConfigCMW_checkDeleteCommand : check the coherency of the delete command
  . unConfigCMW_deleteCommand : delete function
  . unConfigCMW_check : check the coherency of the Front-end config
  . unConfigCMW_set : set Front-end config

        
Internal Functions:
        
Purpose:
This library contains the function used to set the CMW.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
  . unGeneration.cat
  . UNICOS-JCOP framework
  . PVSS version: 3.0
  . operating system: NT and W2000, but tested only under W2000.
  . distributed system: yes.
*/

// Constants
const int CMW_LONGLONG_TYPE = 1002;
const int CMW_LONG_TYPE = 1003;
const int CMW_SHORT_TYPE = 1004;
const int CMW_DOUBLE_TYPE = 1005;
const int CMW_FLOAT_TYPE = 1006;
const int CMW_BOOL_TYPE = 1007;
const int CMW_STRING_TYPE = 1008;
const int CMW_CHAR_TYPE = 1009;
const int CMW_SPONT_INPUT_LLC = 66;
const int CMW_SPONT_INPUT_NOLLC = 2;
const int CMW_SINGLE_QUERY = 3;
const int CMW_POLLING = 4;
const int CMW_OUTPUT = 5;
const string CMW_FILTER_SEPARATOR = "/";

// constant for import of UNCMW
const string UNCMW_DPTYPE = "CMW";
const unsigned UN_CONFIG_CMW_LENGTH = 12;
// 3 mandatory fields: FEtype, FEname, FEappl, 
const unsigned UN_CONFIG_CMW_TYPE = 4;
const unsigned UN_CONFIG_CMW_TRANSFORMATION_TYPE = 5;
const unsigned UN_CONFIG_CMW_GROUP = 6;
const unsigned UN_CONFIG_CMW_CMWDEVICE = 7;
const unsigned UN_CONFIG_CMW_CMWPROPERTY = 8;
const unsigned UN_CONFIG_CMW_CMWTAG = 9;
const unsigned UN_CONFIG_CMW_FILTERNAME = 10;
const unsigned UN_CONFIG_CMW_FILTERVALUE = 11;
const unsigned UN_CONFIG_CMW_LLC = 12;

// constant for UNCMW system integrity
const string c_unSystemIntegrity_UNCMW = "CMW_";
const string CMW_SUBINDEX_SEPARATOR = "#";

//@{

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkDeleteCommand
/**
Purpose: check the coherency of the delete command

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  dsDeleteDps, dyn_string, output, list of dp that will be deleted
  exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkDeleteCommand(dyn_string dsConfig, dyn_string &dsDeleteDps, dyn_string &exceptionInfo)
{
	string sFeName, sPLCSubApplication = "", sObjectType = "", sNumber = "", sDpPlcName, sPlcModPlc;
	dyn_string dsAlarmDps;
	
	if (dynlen(dsConfig) >= 2)				// At least, delete instruction and plc name
	{
// get PLC Name
		sFeName = dsConfig[2];
		if ((sFeName != "") && (dsConfig[1] == UN_DELETE_COMMAND))
		{
			if (dynlen(dsConfig) >= 3)
			{
// get Application
				sPLCSubApplication = dsConfig[3];
			}
			if (dynlen(dsConfig) >= 4)
			{
// get DeviceType
				sObjectType = dsConfig[4];
			}
			if (dynlen(dsConfig) >= 5)
			{
// get device number
				sNumber = dsConfig[5];
			}
			unGenericDpFunctions_getPlcDevices("", sFeName, sPLCSubApplication, sObjectType, sNumber, dsDeleteDps);
// if only delete key word then delete also modbus and systemalarm
			if ((sFeName != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == ""))
			{
// sDpPlc is UNCMW_plcName
				sDpPlcName = c_unSystemIntegrity_UNCMW+sFeName;
				if (dpExists(sDpPlcName))
				{
					dynAppend(dsDeleteDps, sDpPlcName);
					dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*",c_unSystemAlarm_dpType);
					dynAppend(dsDeleteDps, dsAlarmDps);
				}
			}
		}
		else
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkDeleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkDeleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------
//unConfigCMW_check
/**
Purpose: check the coherency of the CMW front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  dsDeleteDps: dyn_string, input, dp to be deleted
  exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
        . PVSS version: 3.0
        . operating system: NT and W2000, but tested only under W2000.
        . distributed system: yes.
*/
unConfigCMW_check(string currentObject, dyn_string currentLineSplit, dyn_string dsDeleteDps, dyn_string &exceptionInfo)
{
	dyn_dyn_string ddsComponentInfo;
//DebugN(currentObject, currentLineSplit);

	switch(currentObject) {
		case UN_PLC_COMMAND:
			unConfigCMW_checkApplication(currentLineSplit, dsDeleteDps, exceptionInfo);
			break;
		default:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_deleteCommand
/**
Purpose: delete function

Parameters:
  dsConfig, dyn_string, input, config dyn_string
  exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_deleteCommand(dyn_string dsConfig, dyn_string &exceptionInfo)
{
	string sPLCSubApplication = "", sObjectType = "", sNumber = "";
	dyn_string dsDpeList, exceptionInfoTemp;
	
	unConfigCMW_checkDeleteCommand(dsConfig, dsDpeList, exceptionInfoTemp);
	if (dynlen(exceptionInfoTemp) > 0)
	{
		dynAppend(exceptionInfo, exceptionInfoTemp);
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_deleteCommand:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		if (dynlen(dsConfig) >= 2)				// At least, delete instruction and plc name
		{
			if (dynlen(dsConfig) >= 3)
			{
				sPLCSubApplication = dsConfig[3];
			}
			if (dynlen(dsConfig) >= 4)
			{
				sObjectType = dsConfig[4];
			}
			if (dynlen(dsConfig) >= 5)
			{
				sNumber = dsConfig[5];
			}

			unGenericDpFunctions_deletePlcDevices(dsConfig[2], sPLCSubApplication, sObjectType, sNumber, "progressBar", exceptionInfo);
			if ((dsConfig[2] != "") && (sPLCSubApplication == "") && (sObjectType == "") && (sNumber == ""))
			{
				unConfigCMW_deleteFrontEnd(dsConfig[2], exceptionInfo);
			}
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
//unConfigCMW_set
/**
Purpose: set CMW front-end config

Parameters:
  currentObject: string, input, type of config
  currentLineSplit: dyn_string, input, config
  exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unConfigCMW_set(string currentObject, dyn_string currentLineSplit, dyn_string &exceptionInfo)
{
	string currentDevice, sPlcName, sPlcType, sAnalogArchive;
	int iPlcNumber, driver;
	bool bEvent16;

//DebugN(currentObject, currentLineSplit);
	switch(currentObject) {
		case UN_PLC_COMMAND:
			unConfigCMW_setApplication(currentLineSplit, exceptionInfo);
			break;
		default:
 			fwException_raise(exceptionInfo, "ERROR", getCatStr("unGeneration","UNKNOWNFUNCTION"), "");
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkApplication
/**
Purpose: check data for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, CMW configuration line in file + additional parameters
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, PLC name
			3. string, PLC subapplication
			4. int, PLC address = PLC number
			(additional parameters)
			5. driver number 
			6. boolean archive name
			7. analog archive name
		dsDeleteDps, dyn_string, input, datapoints that will be deleted before importation
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkApplication(dyn_string dsPLC, dyn_string dsDeleteDps, dyn_string &exceptionInfo)
{
	dyn_string dsPLCLine, dsPLCAdditionalData;
	int i, iRes, iDriverNum;
	string sPLCDp, sHostname, sAliasDp, tempDp;

//DebugN("unConfigCMW_checkApplication", dsPLC, UN_CONFIG_CMW_LENGTH, UN_CONFIG_PLC_ADDITIONAL_LENGTH);
	if (dynlen(dsPLC) == (UN_CONFIG_CMW_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH))
		{
		dsPLCLine = dsPLC;
		for(i=1;i<=UN_CONFIG_PLC_ADDITIONAL_LENGTH;i++)
			{
			dynAppend(dsPLCAdditionalData, dsPLCLine[UN_CONFIG_CMW_LENGTH+1]);
			dynRemove(dsPLCLine,UN_CONFIG_CMW_LENGTH+1);
			}
		unConfigCMW_checkData(dsPLCLine, exceptionInfo);
		unConfigCMW_checkAdditionalData(dsPLCAdditionalData, exceptionInfo);

// first check if the PLC name is correct:
		tempDp = dsPLCLine[UN_CONFIG_PLC_NAME];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration","BADPLCNAME"),"");
			return;
		}

// check if the PLC Application Name is correct:
		tempDp = dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration","BADPLCAPPLICATIONNAME"),"");
			return;
		}

// sPLCDp is UNCMW_plcName
		sPLCDp = getSystemName() +  c_unSystemIntegrity_UNCMW + dsPLCLine[UN_CONFIG_PLC_NAME];
//		sAliasDp = dpAliasToName(dsPLCLine[UN_CONFIG_PLC_NAME]);
		sAliasDp = unGenericDpFunctions_dpAliasToName(dsPLCLine[UN_CONFIG_PLC_NAME]);
//DebugN(sAliasDp, dsPLCLine[UN_CONFIG_PLC_NAME]);

		if (substr(sAliasDp, strlen(sAliasDp) - 1) == ".")
		{
			sAliasDp = substr(sAliasDp, 0, strlen(sAliasDp) - 1);
		}
		if ((sAliasDp != "") && (sAliasDp != sPLCDp))
		{
			if (dynContains(dsDeleteDps, sAliasDp) <= 0)		// Alias exists, is not used by the "CMW" config datapoint and will not be deleted
			{
				fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + dsPLCLine[UN_CONFIG_PLC_NAME] + getCatStr("unGeneration","BADHOSTNAMEALIAS") + sAliasDp,"");
			}
		}
		if (dpExists(sPLCDp) && (dynContains(dsDeleteDps, sPLCDp) <= 0))
		{
			sHostname = unGenericDpFunctions_getAlias(sPLCDp);
			iRes = dpGet(sPLCDp + ".communication.driver_num", iDriverNum);
			if (iRes >= 0)
			{
				if (iDriverNum != dsPLCAdditionalData[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM])
				{
					fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + getCatStr("unGeneration","BADEXISTINGPLCDRIVER")+iDriverNum,"");
				}
				if (sHostname != dsPLCLine[UN_CONFIG_PLC_NAME])
				{
					fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + getCatStr("unGeneration","BADEXISTINGPLCHOSTNAME"),"");
				}
			}
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkApplication: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkData
/**
Purpose: check PLC data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file
			dsPLC must contains :
			1. string, "QUANTUM" or "PREMIUM"
			2. string, name
			3. string, subApplication
			4. int, PLC address = PLC number
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkData(dyn_string dsPLC, dyn_string &exceptionInfo)
{
	unsigned uPLCNumber;
	
	if (dynlen(dsPLC) != UN_CONFIG_CMW_LENGTH)
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkData:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		unConfigGenericFunctions_checkPLCType(dsPLC[UN_CONFIG_PLC_TYPE], exceptionInfo);

		if (strrtrim(dsPLC[UN_CONFIG_PLC_NAME]) == "")
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkData:" + getCatStr("unGeneration","ERRORPLCNAME"),"");
		}
		
		if (strrtrim(dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]) == "")
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkData:" + getCatStr("unGeneration","ERRORSUBAPPLI"),"");
		}			
		// check type
		unConfigCMW_checkType(dsPLC[UN_CONFIG_CMW_TYPE], dsPLC[UN_CONFIG_CMW_GROUP], exceptionInfo);
	
	// check transformation
		unConfigCMW_checkTransformationType(dsPLC[UN_CONFIG_CMW_TRANSFORMATION_TYPE], exceptionInfo);	
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkAdditionalData
/**
Purpose: check additional data for configuration

Parameters:
		dsPLC: dyn_string, input, PLC additional data
			dsPLC must contains :
			1. unsigned, driver number
			2. string, boolean archive
			3. string, analog archive
		exceptionInfo, dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkAdditionalData(dyn_string dsPLC, dyn_string &exceptionInfo)
{
	if (dynlen(dsPLC) != UN_CONFIG_PLC_ADDITIONAL_LENGTH)
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkAdditionalData:" + getCatStr("unGeneration","BADINPUTS"),"");
	}
	else
	{
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkAdditionalData: bool archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkAdditionalData: analog archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
		if (dsPLC[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT] == "")
		{
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_checkAdditionalData: event archive" + getCatStr("unGeneration","ERRORARCHIVEEMPTY"),"");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------
// unConfigCMW_setApplication
/**
Purpose: set PLC configuration for a sub application (not comm)

Parameters:
		dsPLC: dyn_string, input, PLC configuration line in file + additional parameters
			dsPLC must contains :
			(Plc config line)
			1. string, "QUANTUM" or "PREMIUM"
			2. string, PLC name
			3. string, PLC subapplication
			4. int, PLC address = PLC number
			(additional parameters)
			5. driver number 
			6. boolean archive name
			7. analog archive name
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. First line of a generation file must contain the PLC description. Fields are defined in constants at this library beginning
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unConfigCMW_setApplication(dyn_string dsPLCData, dyn_string &exceptionInfo)
{
	int iPLCNumber, iRes, i, position, iDriverNumber;
	string sDpName, addressReference, tempDp;
	dyn_string dsPLC, dsAdditionalParameters, dsSubApplications, dsImportTimes, dsBoolArchives, dsAnalogArchives, exceptionInfoTemp;
	dyn_string dsEventArchives, dsSplit;
	bool bLLC, bGQAllowed;
        dyn_string dsCMWSection;
	
	// Check
	
	if (dynlen(dsPLCData) == (UN_CONFIG_CMW_LENGTH + UN_CONFIG_PLC_ADDITIONAL_LENGTH))
	{
	// 1. Separate config file info and additional parameters
		dsPLC = dsPLCData;
		dynClear(dsAdditionalParameters);
		for(i=1;i<=UN_CONFIG_PLC_ADDITIONAL_LENGTH;i++)
		{
			dynAppend(dsAdditionalParameters, dsPLC[UN_CONFIG_CMW_LENGTH+1]);
			dynRemove(dsPLC,UN_CONFIG_CMW_LENGTH+1);
		}
	
	// check if the PLC Application Name is correct:
		tempDp = dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION];
		if(!unConfigGenericFunctions_nameCheck(tempDp)) {
			fwException_raise(exceptionInfo,"ERROR","unConfigCMW_setApplication: " + dsPLCData[UN_CONFIG_PLC_SUBAPPLICATION] + getCatStr("unGeneration","BADPLCAPPLICATIONNAME"),"");
			return;
		}
	
	// 3. Set PLC configuration in UnCMW
		sDpName = c_unSystemIntegrity_UNCMW + dsPLC[UN_CONFIG_PLC_NAME];
		tempDp = sDpName;
		unConfigGenericFunctions_createDp(sDpName, UNCMW_DPTYPE, exceptionInfoTemp);

		if (dynlen(exceptionInfoTemp) <= 0)
		{
			sDpName = getSystemName() + sDpName;
			unConfigGenericFunctions_setAlias(sDpName, dsPLC[UN_CONFIG_PLC_NAME], exceptionInfo);
			iRes = dpGet(sDpName + ".configuration.subApplications", dsSubApplications,
						 sDpName + ".configuration.importTime", dsImportTimes,
						 sDpName + ".configuration.archive_bool", dsBoolArchives,
						 sDpName + ".configuration.archive_analog", dsAnalogArchives,
						 sDpName + ".configuration.archive_event", dsEventArchives);
			position = dynContains(dsSubApplications, dsPLC[UN_CONFIG_PLC_SUBAPPLICATION]);
			if (position <= 0)
			{
				position = dynlen(dsSubApplications) + 1;
			}
// if the event archive is empty, set it to bool archive
			if(dynlen(dsEventArchives) <= 0)
				dsEventArchives = dsBoolArchives;
				
			dsSubApplications[position] = dsPLC[UN_CONFIG_PLC_SUBAPPLICATION];
			dsImportTimes[position] = getCurrentTime();
			dsBoolArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL];
			dsAnalogArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_ANALOG];
			dsEventArchives[position] = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_EVENT];
	    iRes = dpSetWait(sDpName + ".configuration.importTime", dsImportTimes,
							 sDpName + ".configuration.archive_bool", dsBoolArchives,
							 sDpName + ".configuration.archive_analog", dsAnalogArchives,
							 sDpName + ".configuration.archive_event", dsEventArchives, 
         	  	 sDpName + ".configuration.subApplications",dsSubApplications,
							 sDpName + ".communication.driver_num", dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM]);
			if (iRes < 0)
			{
				fwException_raise(exceptionInfo,"ERROR","unConfigCMW_setApplication: " + getCatStr("unPVSS","DPSETFAILED"),"");
			}	

	// 4. Create _UnSystemAlarm datapoints
			unSystemIntegrity_createSystemAlarm(tempDp,
																					DS_pattern,
																					getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
																					exceptionInfo);
			fwArchive_set(c_unSystemAlarm_dpPattern+DS_pattern+tempDp+".alarm",
						  dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
						  DPATTR_ARCH_PROC_SIMPLESM,
						  DPATTR_COMPARE_OLD_NEW,
						  0,
						  0,
						  exceptionInfo);

			unSystemIntegrity_createSystemAlarm(tempDp,
																					DS_Time_pattern,
																					getCatStr("unSystemIntegrity", "PLC_DS_TIME_DESCRIPTION") + dsPLC[UN_CONFIG_PLC_NAME] + " -> DS driver " + dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM],
																					exceptionInfo);
			fwArchive_set(c_unSystemAlarm_dpPattern+DS_Time_pattern+tempDp+".alarm",
						  dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_ARCHIVE_BOOL],
						  DPATTR_ARCH_PROC_SIMPLESM,
						  DPATTR_COMPARE_OLD_NEW,
						  0,
						  0,
						  exceptionInfo);

			iDriverNumber = dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM];
                        iRes=fwInstallation_getSection( "CMWClient_"+iDriverNumber, dsCMWSection);
                        if(dynlen(dsCMWSection) <= 0)
			  unCMWClient_setDriverConfig(iDriverNumber, exceptionInfo);

// set address config
			dsSplit = strsplit(dsPLC[UN_CONFIG_CMW_LLC], UN_CONFIG_GROUP_SEPARATOR);
			while(dynlen(dsSplit) <=2)
				dynAppend(dsSplit, "NO");
//			if(dsPLC[UN_CONFIG_CMW_LLC] == "TRUE")
			if(dsSplit[1] == "TRUE")
				bLLC = true;
			if(dsSplit[2] == "TRUE")
				bGQAllowed = true;
			unConfigCMW_setAddress(sDpName + ".communication.counter", 
														dsAdditionalParameters[UN_CONFIG_PLC_ADDITIONAL_DRIVER_NUM], 
														dsPLC[UN_CONFIG_CMW_TYPE], 
														dsPLC[UN_CONFIG_CMW_TRANSFORMATION_TYPE],
														dsPLC[UN_CONFIG_CMW_GROUP],
														dsPLC[UN_CONFIG_CMW_FILTERNAME],
														dsPLC[UN_CONFIG_CMW_FILTERVALUE],
														dsPLC[UN_CONFIG_CMW_CMWDEVICE],
														dsPLC[UN_CONFIG_CMW_CMWPROPERTY],
														dsPLC[UN_CONFIG_CMW_CMWTAG],
														bLLC,
														exceptionInfo);
			dpSet(sDpName+".configuration.GQAllowed", bGQAllowed);
		}
		else
		{
			dynAppend(exceptionInfo, exceptionInfoTemp);
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigCMW_setApplication: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_deleteFrontEnd
/**
Purpose: delete Front-end config

Parameters : 
	sPlcName, string, input, plc name
	exceptionInfo, dyn_string, output, for errors
	
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_deleteFrontEnd(string sPlcName, dyn_string& exceptionInfo)
{
	string sDpPlcName, sPlcModPlc="", exceptionText;
	int i, length, iRes;
	dyn_string dsAlarmDps;
	

	sDpPlcName = c_unSystemIntegrity_UNCMW+sPlcName;
	if (!dpExists(sDpPlcName))
		sDpPlcName = "";

	if (sDpPlcName != "")
	{
		if (unGenericDpFunctions_getSystemName(sDpPlcName) == getSystemName())
		{
	// 2. Delete _Un_Plc
			iRes = dpDelete(substr(sDpPlcName, strpos(sDpPlcName, ":") + 1));
			if (iRes < 0)
			{
				fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_deleteFrontEnd: " + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
			}
	// 3. Delete alarm system 
			dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + "*" + substr(sDpPlcName, strpos(sDpPlcName, ":") + 1) + "*",c_unSystemAlarm_dpType);
			length = dynlen(dsAlarmDps);
			for(i=1;i<=length;i++)
			{
				_unSystemIntegrity_modifyApplicationAlertList(dsAlarmDps[i], false, exceptionInfo);
				iRes = dpDelete(dsAlarmDps[i]);
				if (iRes < 0)
				{
					fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_deleteFrontEnd: " + dsAlarmDps[i] + getCatStr("unGeneral", "UNOBJECTNOTDELETED"), "");
				}
			}
			
		}
		else
		{
			fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_deleteFrontEnd: " + sDpPlcName + getCatStr("unGeneral", "UNOBJECTNOTDELETEDBADSYSTEM"), "");
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkType
/**
Purpose:  check the type: spont input, polling, sq, groupname,polling time for CMW front-end type

Parameters:
		sType: string, input, type
		sGroup: string, input, group configuration groupName,polling time
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkType(string sType, string sGroup, dyn_string &exceptionInfo)
{
	dyn_string dsSplit = strsplit(sGroup, UN_CONFIG_GROUP_SEPARATOR);
	unsigned uFormatted;
	int iFormatted;
	
	unConfigGenericFunctions_checkInt(sType, iFormatted, exceptionInfo);
	switch(iFormatted) {
		case DPATTR_ADDR_MODE_INPUT_SPONT:
		case DPATTR_ADDR_MODE_INPUT_SQUERY:
		case DPATTR_ADDR_MODE_OUTPUT_SINGLE:
			break;
		case DPATTR_ADDR_MODE_INPUT_POLL:
			if(dynlen(dsSplit) < 2)
				fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_checkType: bad group conf (groupname,polling time) " +sGroup, "");
			else {
				unConfigGenericFunctions_checkUnsigned(dsSplit[2], uFormatted, exceptionInfo);
				if(strpos(sGroup, " ") > 0)
					fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_checkType: bad group conf (groupname,polling time) " +sGroup, "");
			}
			break;
		default:
			fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_checkType: type (spont, poll, SQ) " +sType, "");
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkTransformationType
/**
Purpose:  check the CMW transformation type

Parameters:
		sTransformation: string, input, CMW transformation type
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkTransformationType(string sTransformation, dyn_string &exceptionInfo)
{
	unsigned uFormatted;
	
	unConfigGenericFunctions_checkUnsigned(sTransformation, uFormatted, exceptionInfo);
	if(dynlen(exceptionInfo) <= 0) {
		switch(uFormatted) {
			case CMW_LONGLONG_TYPE:
			case CMW_LONG_TYPE:
			case CMW_SHORT_TYPE:
			case CMW_DOUBLE_TYPE:
			case CMW_FLOAT_TYPE:
			case CMW_BOOL_TYPE:
			case CMW_STRING_TYPE:
			case CMW_CHAR_TYPE:
				break;
			default:
				switch(uFormatted - CMW_NO_MONITORON){
					case CMW_LONGLONG_TYPE:
					case CMW_LONG_TYPE:
					case CMW_SHORT_TYPE:
					case CMW_DOUBLE_TYPE:
					case CMW_FLOAT_TYPE:
					case CMW_BOOL_TYPE:
					case CMW_STRING_TYPE:
					case CMW_CHAR_TYPE:
						break;
					default:
						fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_checkTransformationType: wrong transformation " +sTransformation, "");
						break;
				}
				break;
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_checkFormatCMWAddress
/**
Purpose:  check the CMW transformation type

Parameters:
		sTransformation: string, input, CMW transformation type
		exceptionInfo: dyn_string, output, for errors

Usage: Internal function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_checkFormatCMWAddress(string sAddress, dyn_string &exceptionInfo)
{
		dyn_string dsComponents;
		
		// DebugN("FUNCTION: unConfigCMW_checkFormatCMWAddress( sAddress="+sAddress+" )");
		
		// Only in case of a defined CMW address (some of them may not be defined at all)
		if (sAddress!="")
			{
				dsComponents=strsplit(sAddress,"$");
				if (dynlen(dsComponents)!=3)
						fwException_raise(exceptionInfo, "ERROR", "unConfigCMW_checkFormatCMWAddress: wrong CMW address  " +sAddress, "");
			}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_addCMWFilter
/**
Purpose: add the CMW filter in the list of filter/replace the value of the selected filter

Parameters:
		sSystemName: string, input, the system name 
		sCMWFilter: string input, the CMW filter name
		sCMWFilterValue: string input, the CMW filter value
		
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_addCMWFilter(string sSystemName, string sCMWFilter, string sCMWFilterValue)
{
	dyn_string dsFilter, dsResult;
	int pos;
	string sFilter = sCMWFilter+ CMW_FILTER_SEPARATOR + sCMWFilterValue;
	
	dpGet(sSystemName+cFilterDpe, dsFilter);
	dsResult = dynPatternMatch(sCMWFilter+ CMW_FILTER_SEPARATOR+"*", dsFilter);
	
//DebugN(sCMWFilter+ CMW_FILTER_SEPARATOR+"*", dsFilter, dsResult);
	if(dynlen(dsResult) > 0) {
		pos = dynContains(dsFilter, dsResult[1]);
//DebugN(pos);
		if(pos > 0) {
			dsFilter[pos] = sFilter;
			dpSet(sSystemName+cFilterDpe, dsFilter);
		}
	}
	else {
		dynAppend(dsFilter, sFilter);
		dpSet(sSystemName+cFilterDpe, dsFilter);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_getCMWFilterValue
/**
Purpose: get the CMW filter value of the given filter reference.

Parameters:
		sSystemName: string, input, the system name 
		sCMWFilterName: string input, the CMW filter name
		sCMWFilter: string output, the CMW filter
		sCMWFilterValue: string output, the CMW filter value
		
Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_getCMWFilterValue(string sSystemName, string sCMWFilterName, string &sCMWFilter, string &sCMWFilterValue)
{
	dyn_string dsFilter, dsResult;
	int pos;
	string sFilter;
	
	dpGet(sSystemName+cFilterDpe, dsFilter);
	dsResult = dynPatternMatch(sCMWFilterName+ CMW_FILTER_SEPARATOR+"*", dsFilter);
	
//DebugN(sCMWFilter+ CMW_FILTER_SEPARATOR+"*", dsFilter, dsResult);
	if(dynlen(dsResult) > 0) {
		pos = dynContains(dsFilter, dsResult[1]);
//DebugN(pos);
		if(pos > 0) {
			sFilter = substr(dsFilter[pos], strpos(dsFilter[pos], CMW_FILTER_SEPARATOR)+1, strlen(dsFilter[pos]));
			sCMWFilter = dsFilter[pos];
		}
	}
	sCMWFilterValue = sFilter;
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_setAddress
/**
Purpose: set the periph address for CMW front-end type

Parameters:
		sDpe: string, input, the data point element on which to set the address
		sDriver: string, input, the driver number
		sType: string, input, the DPE address mode type
		sTransformationType: string, input, the CMW transformation type
		sGroup: string, input, group: groupname,pollingtime
		sFilterName: string, input, the CMW filter name
		sFilterValue: string, input, the CMW filter value
		sCMWDevice: string, input, the CMW device name
		sCMWProperty: string, input, the CMW property name
		sCMWTag: string, input, the CMW tag name
		bLLC: bool, input, true=low level comparison activatd for input/false=low loevel comparison not activated
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_setAddress(string sDpe, string sDriver, string sType, string sTransformationType, string sGroup, string sFilterName, string sFilterValue, 
													string sCMWDevice, string sCMWProperty, string sCMWTag, bool bLLC, dyn_string &exceptionInfo)
{
  int iSubIndex = 0;
  string sTag = sCMWTag;
  dyn_string dsTag=strsplit(sCMWTag, CMW_SUBINDEX_SEPARATOR);
  if(dynlen(dsTag)>1)
  {
    sscanf(dsTag[2], "%d", iSubIndex);
    sTag = dsTag[1];
  }
  unConfigCMW_setAddressAll(sDpe, sDriver, sType, sTransformationType, sGroup, sFilterName, sFilterValue, 
  													sCMWDevice, sCMWProperty, sTag, bLLC, iSubIndex, "", false, exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_setAddressAll
/**
Purpose: set the periph address for CMW front-end type with sub index

Parameters:
		sDpe: string, input, the data point element on which to set the address
		sDriver: string, input, the driver number
		sType: string, input, the DPE address mode type
		sTransformationType: string, input, the CMW transformation type
		sGroup: string, input, group: groupname,pollingtime
		sFilterName: string, input, the CMW filter name
		sFilterValue: string, input, the CMW filter value
		sCMWDevice: string, input, the CMW device name
		sCMWPropertyApm: string, input, the CMW property name with/without APM
		sCMWTag: string, input, the CMW tag name
		bLLC: bool, input, true=low level comparison activatd for input/false=low loevel comparison not activated
  iSubIndex: int, input, subindex.
  sCycle: string, input, cycle name
  bOnChange: bool, input, true=on change/false=no on change
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_setAddressAll(string sDpe, string sDriver, string sType, string sTransformationType, string sGroup, string sFilterName, string sFilterValue, 
													string sCMWDevice, string sCMWPropertyApm, string sCMWTag, bool bLLC, int iSubIndex, string sCycle, bool bOnChange, dyn_string &exceptionInfo)
{
	unsigned uType, uFormatted, uCMWType;
	string sCMWSubscription, sDpNamePoll;
	dyn_string dsSplit;
	string sCompleteCMWProperty,sCMWProperty;
 string sApm;
 
 dsSplit = strsplit(sCMWPropertyApm, ":");
 while(dynlen(dsSplit) <=2)
   dynAppend(dsSplit, "");
 sCMWProperty = dsSplit[1];
 sApm = dsSplit[2];
 
 dsSplit=strsplit(sGroup, UN_CONFIG_GROUP_SEPARATOR);
	unConfigGenericFunctions_checkUnsigned(sType, uType, exceptionInfo);
//DebugN(uType, dsSplit);
	switch(uType) {
		case DPATTR_ADDR_MODE_INPUT_POLL:
			uCMWType = CMW_POLLING;
// create group
			if(dynlen(dsSplit) >=2) {		
				sDpNamePoll = "_poll_"+dsSplit[1]; 
				// DebugN("unConfigGenericFunctions_createDp(sDpName="+sDpNamePoll);
				
				unConfigGenericFunctions_createDp(sDpNamePoll,"_PollGroup" , exceptionInfo);
				dpSetWait(sDpNamePoll + ".SyncMode:_original.._value", 0,
			  								 sDpNamePoll + ".Active:_original.._value",true,
			  								 sDpNamePoll + ".PollInterval:_original.._value",dsSplit[2]);
			}
			break;
		case DPATTR_ADDR_MODE_INPUT_SPONT:
			if(bLLC)
				uCMWType = CMW_SPONT_INPUT_LLC;
			else
				uCMWType = CMW_SPONT_INPUT_NOLLC;
			break;
		case DPATTR_ADDR_MODE_INPUT_SQUERY:
			uCMWType = CMW_SINGLE_QUERY;
			break;
		case DPATTR_ADDR_MODE_OUTPUT_SINGLE:
			uCMWType = CMW_OUTPUT;
			break;
	}
// set address
	unConfigGenericFunctions_checkUnsigned(sTransformationType, uFormatted, exceptionInfo);
// set periph address 
 	if(sFilterName != "") {
		  unConfigCMW_addCMWFilter(getSystemName(), sFilterName, 
																							sFilterValue);
	 }
  // prop:apm:filter:cycle:onChange
  if(bOnChange)
  {
    sCompleteCMWProperty=sCMWProperty+":"+sApm+":"+sFilterName+":"+sCycle+":onChange";
  }
  else if(sCycle != "")
  {
    sCompleteCMWProperty=sCMWProperty+":"+sApm+":"+sFilterName+":"+sCycle;
  }
  else if(sFilterName!="")
  {
    sCompleteCMWProperty=sCMWProperty+":"+sApm+":"+sFilterName;
  }
  else if(sApm!="")
  {
    sCompleteCMWProperty=sCMWProperty+":"+sApm;
  }
  else
    sCompleteCMWProperty=sCMWProperty;
    
	sCMWSubscription = sCMWDevice + "$" +
												sCompleteCMWProperty + "$" +
												sCMWTag;
	fwPeriphAddress_set(sDpe,
																makeDynString(fwPeriphAddress_TYPE_CMW, 
																							sDriver, 
																							sCMWSubscription,
																							uType, 
																							uFormatted, 
																							true,
																							"",
																							"",
																							"",
																							"",
																							bLLC,
																							iSubIndex,
																							0,
																							0,
																							sDpNamePoll,
																							uCMWType),
																	exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_getFrontEndArchiveDp
/**
Purpose: return the list of Dps that wil be archived during the set of the CMW front-end

Parameters:
		sCommand: string, input, the current entry in the import file
		sDp: string, input, the given dp: PLC or SystemAlarm  Dp
		sFEType: string, input, the front-end type
		dsArchiveDp: dyn_string, output, list of archivedDp

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

CMW_getFrontEndArchiveDp(string sCommand, string sDp, string sFEType, dyn_string &dsArchiveDp)
{
	switch(sCommand) {
		case UN_PLC_COMMAND:
			dsArchiveDp = makeDynString(getSystemName()+c_unSystemAlarm_dpPattern+DS_pattern+sDp+".alarm",
																		getSystemName()+c_unSystemAlarm_dpPattern+DS_Time_pattern+sDp+".alarm");
			break;
		default:
			dsArchiveDp = makeDynString();
			break;
	}
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_Com_ExportConfig
/**
Purpose: Export the CMW front-end config 

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/

CMW_Com_ExportConfig(dyn_string dsParam, dyn_string &exceptionInfo)
{
	string sPlcName, sPlcDrv;
	string sDp, sGQAllowed;
	dyn_string dsDpParameters, dsBoolArchive, dsAnalogArchive, dsEventArchive, dsAddDpParameters;

	// DebugN("FUNCTION: CMW_Com_ExportConfig(dsParam= "+dsParam);
	
	if (dynlen(dsParam)==UN_CONFIG_EXPORT_LENGTH)
	{
		sPlcName = dsParam[UN_CONFIG_EXPORT_PLCNAME];
		sDp = c_unSystemIntegrity_UNCMW + sPlcName;
		dpGet(sDp + ".configuration.archive_bool", dsBoolArchive,
									sDp + ".configuration.archive_analog", dsAnalogArchive,
									sDp + ".configuration.archive_event", dsEventArchive,
									sDp + ".communication.driver_num", sPlcDrv,
									sDp + ".configuration.GQAllowed", sGQAllowed);
		TextFieldDriver.text = sPlcDrv;
	
		if (dynlen(dsBoolArchive)>0 && dynlen(dsAnalogArchive)>0 && dynlen(dsEventArchive)>0)
		{
			TextFieldBoolArchive.text = dsBoolArchive[1];
			TextFieldAnalogArchive.text = dsAnalogArchive[1];
			TextFieldEventArchive.text = dsEventArchive[1];
		}

		//Delete
		dynClear(dsDpParameters);
		dynAppend(dsDpParameters, "#" + UN_DELETE_COMMAND);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	
		//PLCCONFIG
		dynClear(dsDpParameters);
		dynAppend(dsDpParameters, UN_PLC_COMMAND);
		dynAppend(dsDpParameters, "CMW");
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
		dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);

		//Get address config		
		unConfigCMW_getAddressConfig(sDp+".communication.counter", dsAddDpParameters);
		dsAddDpParameters[dynlen(dsAddDpParameters)] += UN_CONFIG_GROUP_SEPARATOR+sGQAllowed;
		dynAppend(dsDpParameters, dsAddDpParameters);

// write to file
		unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
	}
	else
		fwException_raise(exceptionInfo, "ERROR", "CMW_Com_ExportConfig(): Wrong Parameters", "");		
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_getAddressConfig
/**
Purpose: return the address config for CMW

Parameters:
		sDpe: string, input, the data point element
		dsResult: dyn_string, output, the config

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

unConfigCMW_getAddressConfig(string sDpe, dyn_string &dsResult)
{
  string sCycle;
  bool bOnChange;
  unConfigCMW_getAddressConfigAll(sDpe, dsResult, sCycle, bOnChange);
}

//------------------------------------------------------------------------------------------------------------------------

// unConfigCMW_getAddressConfigAll
/**
Purpose: return the address config for CMW with cycle name and onChange flag

Parameters:
		sDpe: string, input, the data point element
		dsResult: dyn_string, output, the config
  sCycle: string, output, the cycle name
  bOnChange: bool, output, true=on change/false=no on change

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
unConfigCMW_getAddressConfigAll(string sDpe, dyn_string &dsResult, string &sCycle, bool &bOnChange)
{
	dyn_anytype daAddressConfig;
	bool bConfigExists, bIsActive;
	dyn_string exceptionInfo, dsSplit;
	unsigned uType, uTransf, uPoll;
	string sGroup, sDev, sTag, sProp, sFilterName, sFilterValue, sCMWSubscription, sTemp, sTempValue,pollgroup, sLLC;
 int iSubIndex;
	
	fwPeriphAddress_get(sDpe, bConfigExists, daAddressConfig, bIsActive, exceptionInfo);

	if((bConfigExists) && (dynlen(daAddressConfig) >= fwPeriphAddress_DATATYPE)) 
		{
			uType = daAddressConfig[fwPeriphAddress_DIRECTION];
			uTransf = daAddressConfig[fwPeriphAddress_DATATYPE];
   iSubIndex = daAddressConfig[fwPeriphAddress_CMW_SUBINDEX];
			sCMWSubscription = daAddressConfig[fwPeriphAddress_REFERENCE];
			dsSplit = strsplit(sCMWSubscription, "$");
			if(dynlen(dsSplit) >=3) 
				{
					sDev = dsSplit[1];
					sProp = dsSplit[2];
					sTag = dsSplit[3];
					
					dsSplit = strsplit(sProp, ":");
     while(dynlen(dsSplit)<=4)
       dynAppend(dsSplit, "");
     if(dsSplit[5] == "onChange")
       bOnChange = true;
     else
       bOnChange = false;
     sCycle = dsSplit[4];
//     DebugN("old", sProp);
//     DebugN("new", dsSplit[1]+":"+dsSplit[2]+":"+dsSplit[3]);
     sProp = dsSplit[1];
     if(dsSplit[2] != "")
     { //apm
       if(dsSplit[3] !="") // with filter
         sProp+=":"+dsSplit[2]+":"+dsSplit[3];
       else // no filter
         sProp+=":"+dsSplit[2];
     }
     else
     { // no apm
       if(dsSplit[3] !="") // with filter
         sProp+=":"+dsSplit[2]+":"+dsSplit[3];
     }
//     DebugN("new", sProp);
					dsSplit = strsplit(sProp, ":");
       
					unConfigCMW_getCMWFilterValue(getSystemName(), dsSplit[dynlen(dsSplit)], sTemp, sTempValue);
					if(sTemp!="") 
						{
							sFilterName = dsSplit[dynlen(dsSplit)];
							sFilterValue = sTempValue;
							sProp = substr(sProp, 0, strpos(sProp, sFilterName) -1);
						}
				}
			if(uType == DPATTR_ADDR_MODE_INPUT_POLL) 
				{
					dpGet(sDpe + ":_address.._poll_group",pollgroup);
					if(dpExists(pollgroup)) 
						{
							dpGet(pollgroup+".PollInterval", uPoll);
							sGroup=substr(dpSubStr(pollgroup, DPSUB_DP),strlen("_poll_"), strlen(pollgroup))+UN_CONFIG_GROUP_SEPARATOR+uPoll;
						}
				}
			dpGet(sDpe + ":_address.._lowlevel", sLLC);
		}
	
 if(iSubIndex != 0)
   sTag+=CMW_SUBINDEX_SEPARATOR+iSubIndex;
	dsResult[1] = uType;
	dsResult[2] = uTransf;
	dsResult[3] = sGroup;
	dsResult[4] = sDev;
	dsResult[5] = sProp;
	dsResult[6] = sTag;
	dsResult[7] = sFilterName;
	dsResult[8] = sFilterValue;
	dsResult[9] = sLLC;
}

//------------------------------------------------------------------------------------------------------------------------

// CMW_getFrontEndManagerConfig
/**
Purpose: return the driver config of CMW front-end

Parameters:
		iDriverNumber: 		int, 		input, 	the driver number
		sName: 					string, 		output,	the name to display
		sDriverName: 			string, 		output, 	the manager name
		iManType: 	int, output, the manager type
		sCommandLine: 	int, output, the command line to start the manager

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Win XP.
	. distributed system: yes.
*/

CMW_getFrontEndManagerConfig(int iDriverNumber, string &sName, string &sDriverName, int &iManType, string &sCommandLine)
{
  string sDriverNumber;
  
  sprintf(sDriverNumber, "%02d", iDriverNumber);
  sName = "CMW";
  sDriverName = "WCCOACMWClient";
  iManType = DRIVER_MAN;
  sCommandLine = "-num "+iDriverNumber+" -db CMWClient_Driver_"+sDriverNumber;
}

//------------------------------------------------------------------------------------------------------------------------
#uses "unCMW.ctl"

//@}




