// This control script is used for automatic re-configuration of the LHC Logging DB componenet.
//
// What is it doing:
// 1. re-generate and overwrite all your import files specified for production use .
// 2. check the new import files for errors
// 3. unregister already registered signals (only if option -delete was specified)
// 4. delete hierarchies of already registered signals
// 5. append the new import files for logging
// 6. create test datapoints if they do not exist yet
// 7. trigger a full consinstency check (only if option -consistency was specified)
//
// Before using the script make sure that you have set-up the config file generator properly.
// 
// To start the scipt enter the following command in the console
// WCCOActrl unLHCLoggingDB_AutoImport.ctl -proj PROJECT_NAME -log +stderr -login scada -pass scada [-delete] [-consistency]
// 
// Arguments -login and -pass are mandatory, they correspond to the authentication information used in the import panel.
// Argument -delete is optional, it will unregister all signals before re-import new configuration (you will have a short gap in the logging data)
// Argument -consisntency is optional, it will trigger a full consinstency check after importing the configuration
//
// author:     Josef Hofer EN/ICE/SCD
//
// copyright:  CERN 2015
//
void main (...)
{
  try 
  {      
      DebugTN(__FILE__ + " STARTING MAGIC"); 
   
      string user, pass;
      bool deleteSignals = false;
      bool checkConsistency = false;
      
      va_list args; 
      int len = va_start(args);  // Returns the number of parameters  
      for (int i=1; i<=len; i++) {    
          if (patternMatch("-login",args[i])) { user = args[i+1]; }; // mandatory
          if (patternMatch("-pass",args[i])) { pass = args[i+1]; };  // mandatory
          if (patternMatch("-delete",args[i])) { deleteSignals = true; }; // optional   
          if (patternMatch("-consistency",args[i])) { checkConsistency = true; }; // optional 
      }
      va_end(args); // va_end stops the handling of the list
  
      if(!user) THROW("Error: Specify a user (-login=USERNAME)"); 
      if(!pass) THROW("Error: Specify a pass (-login=PASSWORD)");
      if(deleteSignals) {DebugTN("Info: flag -delete found, all signals will be unregistered first.");}  
      if(checkConsistency) {DebugTN("Info: flag -consistency found, full system integrity check will be executed after import.");}    

      
      connectToDB();
  
      string sDbSchema = getSchema(); 

      string sDataCategory = getDataCategory(sDbSchema, user);

      dyn_string dsConfigurations = getConfigurations();  

      dyn_string dsImportFiles = generateImportFiles(dsConfigurations);
      
      mapping importData = checkImportFiles(dsImportFiles, deleteSignals);

      dyn_string dsSignals = getRegisteredSignals(sDbSchema);         
      if(deleteSignals) { 
          deleteSignals(sDbSchema, dsSignals);
      }      
      deleteHierarchies(sDbSchema, dsSignals);
      
      appendImportData(importData, user, pass, sDataCategory);
   
      createTestDatapoints();

      if(checkConsistency) {
          triggerFullConsistencyCheck();
      }
   
      DebugTN(__FILE__ + " ALL DONE");  
  }
  catch 
  {
    DebugTN(getLastException());        
  }
}

void THROW(string note)
{
    throw(makeError("", PRIO_SEVERE, ERR_PARAM, 54, note));    
}

void connectToDB()
{
  string sDbName, sDbUser, sDbPass;    
  DebugFTN(1, "Connecting to DB");    
  // Get database settings from datapoint 
  if(unLHCLoggingDBSettings_getActiveDbConfig(sDbName, sDbUser, sDbPass) == false) {
    THROW("ERROR", "Loading database settings failed!");       
  }  
  // Establish database connection
  if (unLHCLoggingDBSql_openDbConnection(sDbName, sDbUser, sDbPass) == false) {
    THROW("ERROR", "Login to database: " + sDbUser + "@" + sDbName + " failed!");        
  } 
}

string getSchema()
{  
  DebugFTN(1, "Getting name of the DB-Schema: ");
  dyn_string dsExInfo;
  string sDbSchema = unLHCLoggingDBSettings_getMySystemDbSchema();  
  dyn_string dsDbSchemas = unLHCLoggingDBSql_getDbSchemas(dsExInfo);   
  if(dynlen(dsExInfo)>0) THROW("Error: Getting DB-Schemas");
  if(dynContains(dsDbSchemas, sDbSchema) <= 0) THROW("Error: Getting name of the DB-Schema"); 
  DebugFTN(1, " - " + sDbSchema);  
  return sDbSchema;  
}

string getDataCategory(string sDbSchema, string user)
{ 
  DebugFTN(1, "Getting data categories");
  dyn_string dsExInfo;
  dyn_dyn_mixed ddmDataCategories = unLHCLoggingDBSql_getDataCategoriesForSpecific(sDbSchema, user, dsExInfo);
  if(dynlen(dsExInfo)>0) THROW("Error: Getting data categories");
  if(dynlen(ddmDataCategories[1])!=1) THROW("Error: Wrong number of data categories");
  string sDataCategory = ddmDataCategories[1];
  DebugFTN(1, " - " + sDataCategory); 
  return sDataCategory;
}

dyn_string getConfigurations()
{
  DebugTN("Getting configurations:"); 
  dyn_string dsConfigurations;    
  dsConfigurations = unLHCLoggingDBSettings_getConfigFileGeneratorRules(false, true); 
  if(dynlen(dsConfigurations)<=0) THROW("Error: Getting configurations");                
  DebugTN(" - " + dynStringToString(dsConfigurations, ", ")); 
  return dsConfigurations;  
}

dyn_string generateImportFiles(const dyn_string &dsConfigurations)
{
  DebugTN("Generating import files");
  dyn_string dsImportFiles;
       
  dsImportFiles = unLHCLoggingDBParser_generateImportFiles(dsConfigurations, false);
  if(dynlen(dsImportFiles) != dynlen(dsConfigurations)) THROW("Error: Missmatch between configurations and import files");
  DebugTN(" - " + dynStringToString(dsImportFiles, ", ")); 
  return dsImportFiles;      
}

mapping checkImportFiles(const dyn_string &dsImportFiles, bool deleteSignals)
{
  mapping importData;
 
  for(int i=1; i<=dynlen(dsImportFiles); i++)
  {      
      dyn_string dsErrors, dsWarnings, dsVariableName, dsHierarchy, dsDescription, dsElementId, dsUnit, dsRDBSource, dsDataCategoryId;
      bool bDataCategoryManualInputRequest;
      
      DebugTN("Checking import file: " + dsImportFiles[i]);
      unsigned ret = unLHCLoggingDBParser_getImportData(
              getPath(DATA_REL_PATH) + dsImportFiles[i],
              dsVariableName,
              dsHierarchy,
              dsDescription,
              dsElementId,
              dsUnit,
              dsRDBSource,
              dsDataCategoryId,
              bDataCategoryManualInputRequest,
              dsErrors,
              dsWarnings);
      
      createVerificationReport(dsImportFiles[i], dsErrors, dsWarnings, dynlen(dsVariableName));
      
      if(!deleteSignals) {
          for(int j=1; j<=dynlen(dsWarnings); j++) {
              if(patternMatch("*RENAMED SIGNAL*", dsWarnings[j])) {
                  THROW("RENAMED SIGNAL DETECTED: Send rename request to acc-logging-support, if you want to keep continuous history of these signals, otherwise start script with optional flag -delete, then signals will be unregistered first and then registered with new name");                                   
              }          
          }
      }
      
      int errors = dynlen(dsErrors);
      if(errors>0) {
          DebugTN("Errors: " + errors);
          DebugTN(dsErrors);
      }
      int warnings = dynlen(dsWarnings);
      if(warnings>0) {
          DebugTN("Warnings: " + warnings);
          DebugTN(dsWarnings);
      }
      if(ret == c_unLHCLoggingDBParser_ConfigFile_ErrorNoImport) THROW("Error: Checking import file: " + dsImportFiles[i]); 
 
      mapping data;
      data["VariableName"] = dsVariableName;    
      data["Hierarchy"] = dsHierarchy;
      data["Description"] = dsDescription;
      data["ElementId"] = dsElementId;
      data["Unit"] = dsUnit;
      data["RDBSource"] = dsRDBSource;
      data["DataCategoryId"] = dsDataCategoryId;
      data["DataCategoryManualInputRequest"] = bDataCategoryManualInputRequest;
      
      importData[dsImportFiles[i]] = data;
  }
  return importData;     
}

dyn_string getRegisteredSignals(string sDbSchema)
{
    dyn_string dsExInfo, dsSignals;
    dyn_dyn_mixed data = unLHCLoggingDBSql_getLogVariables(sDbSchema, dsExInfo); 
    if(dynlen(dsExInfo)>0) THROW("Error: Getting registered signals");      
    if(dynlen(data)>0) dsSignals = data[1];        
    return dsSignals;   
}

void deleteSignals(string sDbSchema, const dyn_string &dsSignals)
{
  if(dynlen(dsSignals)>0) {
      DebugTN("Unregistering signals");
      bool success = unLHCLoggingDBSql_unregisterVariablesFromLogging(dsSignals);
      createDeleteSignalsReport(sDbSchema, dsSignals, success);
      if(!success) THROW("Error: Unregistering signals");  
  }      
}

void deleteHierarchies(string sDbSchema, const dyn_string &dsSignals)
{
  if(dynlen(dsSignals)>0) {    
      DebugTN("Deleting hierarchies"); 
      bool success = unLHCLoggingDBSql_deleteHierarchies(dsSignals);
      createDeleteHierarchiesReport(sDbSchema, dsSignals, success);
      if(!success) THROW("Error: Deleting hierarchies");    
  }
}

void appendImportData(mapping &importData, string user, string pass, string sDataCategory)
{
  dyn_string dsImportFiles = mappingKeys(importData);
  for(int i=1; i<=dynlen(dsImportFiles); i++)
  {
      DebugTN("Appending import file: " + dsImportFiles[i]);
      mapping data = importData[dsImportFiles[i]];
      
      if(data["DataCategoryManualInputRequest"] == true) {
          int j_max = dynlen(data["VariableName"]); 
          dynClear(data["DataCategoryId"]); 
          for(int j=1; j<=j_max; j++) {
              dynAppend(data["DataCategoryId"], sDataCategory);
          } 
      }   
      
      bool bSuccess = unLHCLoggingDBSql_registerDataForLogging(
                      data["ElementId"], 
                      data["VariableName"],
                      data["RDBSource"],
                      data["Unit"],
                      data["Description"],
                      data["DataCategoryId"],
                      data["Hierarchy"],
                      user, 
                      pass);  
      
      createRegistrationReport(dsImportFiles[i], dynlen(data["ElementId"]), bSuccess);
                               
      if(!bSuccess) THROW("Error: Appending import file: " + dsImportFiles[i]);      
  }  
}

void createVerificationReport(string importFile, const dyn_string &dsErrors, const dyn_string &dsWarnings, int numberOfSignals, bool showWarnings=true)
{  
  time tNow = getCurrentTime();  
  
  string sReport;
  sReport += "AUTO IMPORT\n\n";
  sReport += "Verification Report\n";
  sReport += "===================\n";
  sReport += "\n";
  sReport += "Config file: " + importFile + "\n";
  sReport += "Report created: "+ (string)tNow + "\n";
  sReport += "\n";
  sReport += "Elements: " + numberOfSignals + "\n";
  sReport += "Warnings: " + dynlen(dsWarnings) + "\n";
  sReport += "Errors: " + dynlen(dsErrors) + "\n";
  sReport += "\n";
  
  for(int i=1; i<=dynlen(dsErrors); i++) {
    sReport += dsErrors[i] + "\n"; 
  }
  sReport += "\n";
  
  if(showWarnings) {
      for(int i=1; i<=dynlen(dsWarnings); i++) { 
        sReport += dsWarnings[i] + "\n";   
    }
    sReport += "\n";
  } 
    
  string ff = getPath(LOG_REL_PATH) + "unLHCLoggingDB_Check" + formatTime("_%Y_%m_%d_%H_%M_%S", tNow) + ".txt";
  file reportFile = fopen(ff, "w");
  fputs(sReport, reportFile);
  fflush(reportFile);        
  fclose(reportFile);   
}

void createRegistrationReport(string importFile, int numberOfSignals, bool bSuccess)
{  
  time tNow = getCurrentTime(); 
  
  string sReport;  
  sReport += "AUTO IMPORT\n\n";
  sReport += "Registration Report"+ "\n";
  sReport += "==================="+ "\n"; 
  sReport += "\n"; 
  sReport += "Config file: " + importFile + "\n";
  sReport += "Report created: "+ (string)tNow + "\n"; 
  sReport += "\n"; 
  sReport += "Elements: " + numberOfSignals + "\n";
  sReport += "\n";  
  if (bSuccess) {
    sReport += "Registering data was successful" + "\n";      
  } else {
    sReport += "Registering data failed" + "\n";    
    sReport += "\n"; 
    sReport += "Possible causes:" + "\n";   
    sReport += "\n"; 
    sReport += "- Renaming signals: " + "\n" +
               "  + When signals are renamed send a request to acc-logging-support first, " + "\n" +
               "    otherwise the import might fail! " + "\n" +
               "  + The only way to force an import, is to 'Delete' those signals first and then 'Import' the file, " + "\n" +
               "    but then a new signal will be created and the continuos history will be lost, " + "\n" +
               "    because the old signal will still remain in timber." + "\n";   
    sReport += "- Invalid hierarchy: " + "\n" +
               "  + Ensure to have uniform hierarchies (e.g. 'ATLAS' and 'Atlas' in the same file will not work)" + "\n";   
  }  
  sReport += "\n";   
    
  string ff = getPath(LOG_REL_PATH) + "unLHCLoggingDB_Import" + formatTime("_%Y_%m_%d_%H_%M_%S", tNow) + ".txt";
  file reportFile = fopen(ff, "w");
  fputs(sReport, reportFile);
  fflush(reportFile);        
  fclose(reportFile);  
}

void createDeleteSignalsReport(string sDbSchema, const dyn_string &dsVarNames, bool bSuccess)
{   
  time tNow = getCurrentTime();  

  string sReport; 
  sReport += "AUTO IMPORT\n\n"; 
  sReport += "Delete Signals Report" + "\n";
  sReport += "=====================" + "\n"; 
  sReport += "\n"; 
  sReport += "Report created: "+ (string)tNow  + "\n"; 
  sReport += "\n";    
  if(bSuccess) {
    sReport += "Success: Following signals of DB-Schema '"+sDbSchema+"' have been deleted: \n\n";  
    for(int i=1; i<=dynlen(dsVarNames); i++) {
        sReport += dsVarNames[i] + "\n";
    }       
  }else {
    sReport +="Error: Deleting signals of DB-Schema '" + sDbSchema + "' failed\n";   
  }
  
  string ff = getPath(LOG_REL_PATH) + "unLHCLoggingDB_Delete" + formatTime("_%Y_%m_%d_%H_%M_%S", tNow) + ".txt";
  file reportFile = fopen(ff, "w");  
  fputs(sReport, reportFile);
  fflush(reportFile);        
  fclose(reportFile);    
}

void createDeleteHierarchiesReport(string sDbSchema, const dyn_string &dsVarNames, bool bSuccess)
{   
  time tNow = getCurrentTime();  

  string sReport;  
  sReport += "AUTO IMPORT\n\n";
  sReport += "Delete Hierarchies Report" + "\n";
  sReport += "=========================" + "\n"; 
  sReport += "\n"; 
  sReport += "Report created: "+ (string)tNow  + "\n"; 
  sReport += "\n"; 
  if(bSuccess) {
    sReport += "Success: Hierarchies of the following signals of DB-Schema '"+sDbSchema+"' have been deleted\n\n";  
    for(int i=1; i<=dynlen(dsVarNames); i++) {
        sReport += dsVarNames[i] + "\n";
    }       
  }else {
    sReport +="Error: Deleting hierarchies of DB-Schema '" + sDbSchema + "' failed\n";   
  }
    
  string ff = getPath(LOG_REL_PATH) + "unLHCLoggingDB_DeleteHierarchies" + formatTime("_%Y_%m_%d_%H_%M_%S", tNow) + ".txt";
  file reportFile = fopen(ff, "w");  
  fputs(sReport, reportFile);
  fflush(reportFile);        
  fclose(reportFile);    
}

void createTestDatapoints()
{
  DebugTN("Creating Test Datapoints");
  dpSetWait(getSystemName()+LHCLoggingDB_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".interface.command", c_SysInt_LHCLoggingDB_CreateTestDps);	
  delay(10); 	        
}

void triggerFullConsistencyCheck()
{    
  dyn_string dsConfigurations = unLHCLoggingDBSettings_getConfigFileGeneratorRules(false, true, getSystemName()); 
  unSystemIntegrity_LHCLoggingDB_triggerFullConsistencyCheck(
        getSystemName(),     // system to check
        dsConfigurations, // list of configurations to check
        false,            // dont generate temp import files
        true,             // trigger system integrity error
        false);           // save report in datapoint
}
